import Foundation

/// A screen container that can store navigation data.
///
/// This protocol is used in screen decorators, which can generate data
/// that needs to be stored in memory until the container itself is released.
///
/// This protocol is already implemented by the `UIViewController` class and all its subclasses.
/// The default implementation for `NSObject` subclasses uses the `objc_setAssociatedObject` method
/// to associate the payload with the container.
///
/// - SeeAlso: `ScreenContainer`
public protocol ScreenPayloadedContainer: ScreenContainer {

    /// Screen payload.
    ///
    /// This object stores the data generated by navigation.
    ///
    /// - SeeAlso: `ScreenPayload`
    var screenPayload: ScreenPayload { get }
}
